@@ -39,9 +39,7 @@ from mlflow.utils.requirements_utils import _get_pinned_requirement
 from mlflow.utils.file_utils import write_to
 from mlflow.utils.docstring_utils import format_docstring, LOG_MODEL_PARAM_DOCS
 from mlflow.utils.model_utils import _get_flavor_configuration
-from mlflow.utils.annotations import experimental
 from mlflow.utils.autologging_utils import (
-    try_mlflow_log,
     log_fn_args_as_params,
     safe_patch,
     batch_metrics_logger,
@@ -83,7 +81,7 @@ def save_model(
     input_example: ModelInputExample = None,
     pip_requirements=None,
     extra_pip_requirements=None,
-    **kwargs
+    **kwargs,
 ):
     """
     Save a fastai Learner to a path on the local file system.
@@ -179,13 +177,17 @@ def save_model(
             # To ensure `_load_pyfunc` can successfully load the model during the dependency
             # inference, `mlflow_model.save` must be called beforehand to save an MLmodel file.
             inferred_reqs = mlflow.models.infer_pip_requirements(
-                path, FLAVOR_NAME, fallback=default_reqs,
+                path,
+                FLAVOR_NAME,
+                fallback=default_reqs,
             )
             default_reqs = sorted(set(inferred_reqs).union(default_reqs))
         else:
             default_reqs = None
         conda_env, pip_requirements, pip_constraints = _process_pip_requirements(
-            default_reqs, pip_requirements, extra_pip_requirements,
+            default_reqs,
+            pip_requirements,
+            extra_pip_requirements,
         )
     else:
         conda_env, pip_requirements, pip_constraints = _process_conda_env(conda_env)
@@ -212,7 +214,7 @@ def log_model(
     await_registration_for=DEFAULT_AWAIT_MAX_SLEEP_SECONDS,
     pip_requirements=None,
     extra_pip_requirements=None,
-    **kwargs
+    **kwargs,
 ):
     """
     Log a fastai model as an MLflow artifact for the current run.
@@ -250,6 +252,8 @@ def log_model(
                             waits for five minutes. Specify 0 or None to skip waiting.
     :param pip_requirements: {{ pip_requirements }}
     :param extra_pip_requirements: {{ extra_pip_requirements }}
+    :return: A :py:class:`ModelInfo <mlflow.models.model.ModelInfo>` instance that contains the
+             metadata of the logged model.
 
     .. code-block:: python
         :caption: Example
@@ -285,7 +289,7 @@ def log_model(
 
         artifacts: ['model/MLmodel', 'model/conda.yaml', 'model/model.fastai']
     """
-    Model.log(
+    return Model.log(
         artifact_path=artifact_path,
         flavor=mlflow.fastai,
         registered_model_name=registered_model_name,
@@ -313,7 +317,7 @@ class _FastaiModelWrapper:
     def predict(self, dataframe):
         dl = self.learner.dls.test_dl(dataframe)
         preds, _ = self.learner.get_preds(dl=dl)
-        return pd.DataFrame(map(np.array, preds.numpy()), columns=["predictions"])
+        return pd.Series(map(np.array, preds.numpy())).to_frame("predictions")
 
 
 def _load_pyfunc(path):
@@ -325,7 +329,7 @@ def _load_pyfunc(path):
     return _FastaiModelWrapper(_load_model(path))
 
 
-def load_model(model_uri):
+def load_model(model_uri, dst_path=None):
     """
     Load a fastai model from a local file or a run.
 
@@ -339,6 +343,9 @@ def load_model(model_uri):
                       For more information about supported URI schemes, see
                       `Referencing Artifacts <https://www.mlflow.org/docs/latest/tracking.html#
                       artifact-locations>`_.
+    :param dst_path: The local filesystem path to which to download the model artifact.
+                     This directory must already exist. If unspecified, a local output
+                     path will be created.
 
     :return: A fastai model (an instance of `fastai.Learner`_).
 
@@ -360,13 +367,12 @@ def load_model(model_uri):
         loaded_model = mlflow.fastai.load_model(model_uri)
         results = loaded_model.predict(predict_data)
     """
-    local_model_path = _download_artifact_from_uri(artifact_uri=model_uri)
+    local_model_path = _download_artifact_from_uri(artifact_uri=model_uri, output_path=dst_path)
     flavor_conf = _get_flavor_configuration(model_path=local_model_path, flavor_name=FLAVOR_NAME)
     model_file_path = os.path.join(local_model_path, flavor_conf.get("data", "model.fastai"))
     return _load_model(path=model_file_path)
 
 
-@experimental
 @autologging_integration(FLAVOR_NAME)
 def autolog(
     log_models=True,
@@ -374,6 +380,7 @@ def autolog(
     exclusive=False,
     disable_for_unsupported_versions=False,
     silent=False,
+    registered_model_name=None,
 ):  # pylint: disable=unused-argument
     """
     Enable automatic logging from Fastai to MLflow.
@@ -398,6 +405,9 @@ def autolog(
     :param silent: If ``True``, suppress all event logs and warnings from MLflow during Fastai
                    autologging. If ``False``, show all events and warnings during Fastai
                    autologging.
+    :param registered_model_name: If given, each time a model is trained, it is registered as a
+                                  new model version of the registered model with this name.
+                                  The registered model is created if it does not already exist.
 
     .. code-block:: python
         :caption: Example
@@ -473,7 +483,9 @@ def autolog(
         from mlflow.fastai.callback import __MlflowFastaiCallback
 
         return __MlflowFastaiCallback(
-            metrics_logger=metrics_logger, log_models=log_models, is_fine_tune=is_fine_tune,
+            metrics_logger=metrics_logger,
+            log_models=log_models,
+            is_fine_tune=is_fine_tune,
         )
 
     def _find_callback_of_type(callback_type, callbacks):
@@ -491,7 +503,7 @@ def autolog(
                     "early_stop_patience": callback.patience,
                     "early_stop_comp": callback.comp.__name__,
                 }
-                try_mlflow_log(mlflow.log_params, earlystopping_params)
+                mlflow.log_params(earlystopping_params)
             except Exception:  # pylint: disable=W0703
                 return
 
@@ -506,8 +518,8 @@ def autolog(
         infos = layer_info(learner, *xb)
         bs = find_bs(xb)
         inp_sz = _print_shapes(map(lambda x: x.shape, xb), bs)
-        try_mlflow_log(mlflow.log_param, "input_size", inp_sz)
-        try_mlflow_log(mlflow.log_param, "num_layers", len(infos))
+        mlflow.log_param("input_size", inp_sz)
+        mlflow.log_param("num_layers", len(infos))
 
         summary = module_summary(learner, *xb)
 
@@ -520,7 +532,7 @@ def autolog(
             summary_file = os.path.join(tempdir, "module_summary.txt")
             with open(summary_file, "w") as f:
                 f.write(summary)
-            try_mlflow_log(mlflow.log_artifact, local_path=summary_file)
+            mlflow.log_artifact(local_path=summary_file)
         finally:
             shutil.rmtree(tempdir)
 
@@ -546,7 +558,7 @@ def autolog(
 
                 # Log information regarding model and data without bar and print-out
                 with self.no_bar(), self.no_logging():
-                    try_mlflow_log(_log_model_info, learner=self)
+                    _log_model_info(learner=self)
 
                 mlflowFastaiCallback = getFastaiCallback(
                     metrics_logger=metrics_logger, is_fine_tune=is_fine_tune
