@@ -6,6 +6,7 @@ import threading
 import math
 import sqlalchemy
 import sqlalchemy.sql.expression as sql
+from sqlalchemy.future import select
 
 from mlflow.entities.lifecycle_stage import LifecycleStage
 from mlflow.store.tracking import SEARCH_MAX_RESULTS_THRESHOLD
@@ -224,7 +225,7 @@ class SqlAlchemyStore(AbstractStore):
     def _get_artifact_location(self, experiment_id):
         return append_to_uri_path(self.artifact_root_uri, str(experiment_id))
 
-    def create_experiment(self, name, artifact_location=None):
+    def create_experiment(self, name, artifact_location=None, tags=None):
         if name is None or name == "":
             raise MlflowException("Invalid experiment name", INVALID_PARAMETER_VALUE)
 
@@ -235,6 +236,9 @@ class SqlAlchemyStore(AbstractStore):
                     lifecycle_stage=LifecycleStage.ACTIVE,
                     artifact_location=artifact_location,
                 )
+                experiment.tags = (
+                    [SqlExperimentTag(key=tag.key, value=tag.value) for tag in tags] if tags else []
+                )
                 session.add(experiment)
                 if not artifact_location:
                     # this requires a double write. The first one to generate an autoincrement-ed ID
@@ -307,7 +311,10 @@ class SqlAlchemyStore(AbstractStore):
             return PagedList(experiments, None)
 
     def list_experiments(
-        self, view_type=ViewType.ACTIVE_ONLY, max_results=None, page_token=None,
+        self,
+        view_type=ViewType.ACTIVE_ONLY,
+        max_results=None,
+        page_token=None,
     ):
         """
         :param view_type: Qualify requested type of experiments.
@@ -433,10 +440,7 @@ class SqlAlchemyStore(AbstractStore):
                 lifecycle_stage=LifecycleStage.ACTIVE,
             )
 
-            tags_dict = {}
-            for tag in tags:
-                tags_dict[tag.key] = tag.value
-            run.tags = [SqlTag(key=key, value=value) for key, value in tags_dict.items()]
+            run.tags = [SqlTag(key=tag.key, value=tag.value) for tag in tags] if tags else []
             self._save_to_db(objs=run, session=session)
 
             return run.to_mlflow_entity()
@@ -472,13 +476,13 @@ class SqlAlchemyStore(AbstractStore):
                  run attributes when fetching a run: ``latest_metrics``, ``params``, and ``tags``.
         """
         return [
-            # Use a subquery load rather than a joined load in order to minimize the memory overhead
-            # of the eager loading procedure. For more information about relationship loading
-            # techniques, see https://docs.sqlalchemy.org/en/13/orm/
+            # Use a select in load rather than a joined load in order to minimize the memory
+            # overhead of the eager loading procedure. For more information about relationship
+            # loading techniques, see https://docs.sqlalchemy.org/en/13/orm/
             # loading_relationships.html#relationship-loading-techniques
-            sqlalchemy.orm.subqueryload(SqlRun.latest_metrics),
-            sqlalchemy.orm.subqueryload(SqlRun.params),
-            sqlalchemy.orm.subqueryload(SqlRun.tags),
+            sqlalchemy.orm.selectinload(SqlRun.latest_metrics),
+            sqlalchemy.orm.selectinload(SqlRun.params),
+            sqlalchemy.orm.selectinload(SqlRun.tags),
         ]
 
     def _check_run_is_active(self, run):
@@ -769,31 +773,31 @@ class SqlAlchemyStore(AbstractStore):
             # ``run.to_mlflow_entity()``, so eager loading helps avoid additional database queries
             # that are otherwise executed at attribute access time under a lazy loading model.
             parsed_filters = SearchUtils.parse_search_filter(filter_string)
-            parsed_orderby, sorting_joins = _get_orderby_clauses(order_by, session)
+            cases_orderby, parsed_orderby, sorting_joins = _get_orderby_clauses(order_by, session)
 
-            query = session.query(SqlRun)
+            stmt = select(SqlRun, *cases_orderby)
             for j in _get_sqlalchemy_filter_clauses(parsed_filters, session):
-                query = query.join(j)
+                stmt = stmt.join(j)
             # using an outer join is necessary here because we want to be able to sort
             # on a column (tag, metric or param) without removing the lines that
             # do not have a value for this column (which is what inner join would do)
             for j in sorting_joins:
-                query = query.outerjoin(j)
+                stmt = stmt.outerjoin(j)
 
             offset = SearchUtils.parse_start_offset_from_page_token(page_token)
-            queried_runs = (
-                query.distinct()
+            stmt = (
+                stmt.distinct()
                 .options(*self._get_eager_run_query_options())
                 .filter(
                     SqlRun.experiment_id.in_(experiment_ids),
                     SqlRun.lifecycle_stage.in_(stages),
-                    *_get_attributes_filtering_clauses(parsed_filters)
+                    *_get_attributes_filtering_clauses(parsed_filters),
                 )
                 .order_by(*parsed_orderby)
                 .offset(offset)
                 .limit(max_results)
-                .all()
             )
+            queried_runs = session.execute(stmt).scalars(SqlRun).all()
 
             runs = [run.to_mlflow_entity() for run in queried_runs]
             next_page_token = compute_next_token(len(runs))
@@ -848,7 +852,9 @@ def _get_attributes_filtering_clauses(parsed):
         key_name = sql_statement.get("key")
         value = sql_statement.get("value")
         comparator = sql_statement.get("comparator").upper()
-        if SearchUtils.is_attribute(key_type, comparator):
+        if SearchUtils.is_string_attribute(
+            key_type, key_name, comparator
+        ) or SearchUtils.is_numeric_attribute(key_type, key_name, comparator):
             # key_name is guaranteed to be a valid searchable attribute of entities.RunInfo
             # by the call to parse_search_filter
             attribute = getattr(SqlRun, SqlRun.get_attribute_name(key_name))
@@ -874,7 +880,9 @@ def _to_sqlalchemy_filtering_statement(sql_statement, session):
         entity = SqlParam
     elif SearchUtils.is_tag(key_type, comparator):
         entity = SqlTag
-    elif SearchUtils.is_attribute(key_type, comparator):
+    elif SearchUtils.is_string_attribute(
+        key_type, key_name, comparator
+    ) or SearchUtils.is_numeric_attribute(key_type, key_name, comparator):
         return None
     else:
         raise MlflowException(
@@ -913,13 +921,16 @@ def _get_orderby_clauses(order_by_list, session):
     ordering_joins = []
     clause_id = 0
     observed_order_by_clauses = set()
+    select_clauses = []
     # contrary to filters, it is not easily feasible to separately handle sorting
     # on attributes and on joined tables as we must keep all clauses in the same order
     if order_by_list:
         for order_by_clause in order_by_list:
             clause_id += 1
             (key_type, key, ascending) = SearchUtils.parse_order_by_for_search_runs(order_by_clause)
-            if SearchUtils.is_attribute(key_type, "="):
+            if SearchUtils.is_string_attribute(
+                key_type, key, "="
+            ) or SearchUtils.is_numeric_attribute(key_type, key, "="):
                 order_value = getattr(SqlRun, SqlRun.get_attribute_name(key))
             else:
                 if SearchUtils.is_metric(key_type, "="):  # any valid comparator
@@ -947,24 +958,26 @@ def _get_orderby_clauses(order_by_list, session):
             # same main query, the CASE WHEN columns need to have unique names to
             # avoid ambiguity
             if SearchUtils.is_metric(key_type, "="):
-                clauses.append(
-                    sql.case(
-                        [
-                            # Ideally the use of "IS" is preferred here but owing to sqlalchemy
-                            # translation in MSSQL we are forced to use "=" instead.
-                            # These 2 options are functionally identical / unchanged because
-                            # the column (is_nan) is not nullable. However it could become an issue
-                            # if this precondition changes in the future.
-                            (subquery.c.is_nan == sqlalchemy.true(), 1),
-                            (order_value.is_(None), 1),
-                        ],
-                        else_=0,
-                    ).label("clause_%s" % clause_id)
-                )
+                case = sql.case(
+                    [
+                        # Ideally the use of "IS" is preferred here but owing to sqlalchemy
+                        # translation in MSSQL we are forced to use "=" instead.
+                        # These 2 options are functionally identical / unchanged because
+                        # the column (is_nan) is not nullable. However it could become an issue
+                        # if this precondition changes in the future.
+                        (subquery.c.is_nan == sqlalchemy.true(), 1),
+                        (order_value.is_(None), 1),
+                    ],
+                    else_=0,
+                ).label("clause_%s" % clause_id)
+
             else:  # other entities do not have an 'is_nan' field
-                clauses.append(
-                    sql.case([(order_value.is_(None), 1)], else_=0).label("clause_%s" % clause_id)
+                case = sql.case([(order_value.is_(None), 1)], else_=0).label(
+                    "clause_%s" % clause_id
                 )
+            clauses.append(case.name)
+            select_clauses.append(case)
+            select_clauses.append(order_value)
 
             if (key_type, key) in observed_order_by_clauses:
                 raise MlflowException(
@@ -980,4 +993,4 @@ def _get_orderby_clauses(order_by_list, session):
     if (SearchUtils._ATTRIBUTE_IDENTIFIER, SqlRun.start_time.key) not in observed_order_by_clauses:
         clauses.append(SqlRun.start_time.desc())
     clauses.append(SqlRun.run_uuid)
-    return clauses, ordering_joins
+    return select_clauses, clauses, ordering_joins
